@@ -49,9 +49,19 @@
 #include "psy.h"
 #include "rate.h"
 #include "stack_alloc.h"
+#include "mathops.h"
 
 static const celt_word16_t preemph = QCONST16(0.8f,15);
 
+#ifdef FIXED_POINT
+static const celt_word16_t transientWindow[16] = {
+     279,  1106,  2454,  4276,  6510,  9081, 11900, 14872,
+   17896, 20868, 23687, 26258, 28492, 30314, 31662, 32489};
+#else
+static const float transientWindow[16] = {
+   0.0085135, 0.0337639, 0.0748914, 0.1304955, 0.1986827, 0.2771308, 0.3631685, 0.4538658,
+   0.5461342, 0.6368315, 0.7228692, 0.8013173, 0.8695045, 0.9251086, 0.9662361, 0.9914865};
+#endif
 
 /** Encoder state 
  @brief Encoder state
@@ -73,9 +83,13 @@ struct CELTEncoder {
    celt_sig_t *out_mem;
 
    celt_word16_t *oldBandE;
+#ifdef EXP_PSY
+   celt_word16_t *psy_mem;
+   struct PsyDecay psy;
+#endif
 };
 
-CELTEncoder EXPORT *celt_encoder_create(const CELTMode *mode)
+CELTEncoder *celt_encoder_create(const CELTMode *mode)
 {
    int N, C;
    CELTEncoder *st;
@@ -103,10 +117,15 @@ CELTEncoder EXPORT *celt_encoder_create(const CELTMode *mode)
    st->preemph_memE = (celt_word16_t*)celt_alloc(C*sizeof(celt_word16_t));;
    st->preemph_memD = (celt_sig_t*)celt_alloc(C*sizeof(celt_sig_t));;
 
+#ifdef EXP_PSY
+   st->psy_mem = celt_alloc(MAX_PERIOD*sizeof(celt_word16_t));
+   psydecay_init(&st->psy, MAX_PERIOD/2, st->mode->Fs);
+#endif
+
    return st;
 }
 
-void EXPORT celt_encoder_destroy(CELTEncoder *st)
+void celt_encoder_destroy(CELTEncoder *st)
 {
    if (st == NULL)
    {
@@ -126,6 +145,11 @@ void EXPORT celt_encoder_destroy(CELTEncoder *st)
    celt_free(st->preemph_memE);
    celt_free(st->preemph_memD);
    
+#ifdef EXP_PSY
+   celt_free (st->psy_mem);
+   psydecay_clear(&st->psy);
+#endif
+   
    celt_free(st);
 }
 
@@ -141,17 +165,69 @@ static inline celt_int16_t SIG2INT16(celt_sig_t x)
 #endif
 }
 
-/** Apply window and compute the MDCT for all sub-frames and all channels in a frame */
-static void compute_mdcts(const CELTMode *mode, const celt_word16_t * restrict window, celt_sig_t * restrict in, celt_sig_t * restrict out)
+static int transient_analysis(celt_word32_t *in, int len, int C, int *transient_time, int *transient_shift)
 {
-   const mdct_lookup *lookup = MDCT(mode);
-   const int N = FRAMESIZE(mode);
-   const int C = CHANNELS(mode);
-   const int overlap = OVERLAP(mode);
-   if (C==1)
+   int c, i, n;
+   celt_word32_t ratio;
+   /* FIXME: Remove the floats here */
+   VARDECL(celt_word32_t, begin);
+   SAVE_STACK;
+   ALLOC(begin, len, celt_word32_t);
+   for (i=0;i<len;i++)
+      begin[i] = EXTEND32(ABS16(SHR32(in[C*i],SIG_SHIFT)));
+   for (c=1;c<C;c++)
    {
-      mdct_forward(lookup, in, out, window, overlap);
+      for (i=0;i<len;i++)
+         begin[i] = MAX32(begin[i], EXTEND32(ABS16(SHR32(in[C*i+c],SIG_SHIFT))));
+   }
+   for (i=1;i<len;i++)
+      begin[i] = MAX32(begin[i-1],begin[i]);
+   n = -1;
+   for (i=8;i<len-8;i++)
+   {
+      if (begin[i] < MULT16_32_Q15(QCONST16(.2f,15),begin[len-1]))
+         n=i;
+   }
+   if (n<32)
+   {
+      n = -1;
+      ratio = 0;
    } else {
+      ratio = DIV32(begin[len-1],1+begin[n-16]);
+   }
+   /*printf ("%d %f\n", n, ratio*ratio);*/
+   if (ratio < 0)
+      ratio = 0;
+   if (ratio > 1000)
+      ratio = 1000;
+   ratio *= ratio;
+   if (ratio < 50)
+      *transient_shift = 0;
+   else if (ratio < 256)
+      *transient_shift = 1;
+   else if (ratio < 4096)
+      *transient_shift = 2;
+   else
+      *transient_shift = 3;
+   *transient_time = n;
+   
+   RESTORE_STACK;
+   return ratio > 20;
+}
+
+/** Apply window and compute the MDCT for all sub-frames and all channels in a frame */
+static void compute_mdcts(const CELTMode *mode, int shortBlocks, celt_sig_t * restrict in, celt_sig_t * restrict out)
+{
+   const int C = CHANNELS(mode);
+   if (C==1 && !shortBlocks)
+   {
+      const mdct_lookup *lookup = MDCT(mode);
+      const int overlap = OVERLAP(mode);
+      mdct_forward(lookup, in, out, mode->window, overlap);
+   } else if (!shortBlocks) {
+      const mdct_lookup *lookup = MDCT(mode);
+      const int overlap = OVERLAP(mode);
+      const int N = FRAMESIZE(mode);
       int c;
       VARDECL(celt_word32_t, x);
       VARDECL(celt_word32_t, tmp);
@@ -163,30 +239,56 @@ static void compute_mdcts(const CELTMode *mode, const celt_word16_t * restrict w
          int j;
          for (j=0;j<N+overlap;j++)
             x[j] = in[C*j+c];
-         mdct_forward(lookup, x, tmp, window, overlap);
+         mdct_forward(lookup, x, tmp, mode->window, overlap);
          /* Interleaving the sub-frames */
          for (j=0;j<N;j++)
             out[C*j+c] = tmp[j];
       }
       RESTORE_STACK;
+   } else {
+      const mdct_lookup *lookup = &mode->shortMdct;
+      const int overlap = mode->shortMdctSize;
+      const int N = mode->shortMdctSize;
+      int b, c;
+      VARDECL(celt_word32_t, x);
+      VARDECL(celt_word32_t, tmp);
+      SAVE_STACK;
+      ALLOC(x, N+overlap, celt_word32_t);
+      ALLOC(tmp, N, celt_word32_t);
+      for (c=0;c<C;c++)
+      {
+         int B = mode->nbShortMdcts;
+         for (b=0;b<B;b++)
+         {
+            int j;
+            for (j=0;j<N+overlap;j++)
+               x[j] = in[C*(b*N+j)+c];
+            mdct_forward(lookup, x, tmp, mode->window, overlap);
+            /* Interleaving the sub-frames */
+            for (j=0;j<N;j++)
+               out[C*(j*B+b)+c] = tmp[j];
+         }
+      }
+      RESTORE_STACK;
    }
 }
 
 /** Compute the IMDCT and apply window for all sub-frames and all channels in a frame */
-static void compute_inv_mdcts(const CELTMode *mode, const celt_word16_t * restrict window, celt_sig_t *X, celt_sig_t * restrict out_mem)
+static void compute_inv_mdcts(const CELTMode *mode, int shortBlocks, celt_sig_t *X, int transient_time, int transient_shift, celt_sig_t * restrict out_mem)
 {
    int c, N4;
    const int C = CHANNELS(mode);
-   const mdct_lookup *lookup = MDCT(mode);
    const int N = FRAMESIZE(mode);
    const int overlap = OVERLAP(mode);
    N4 = (N-overlap)>>1;
    for (c=0;c<C;c++)
    {
       int j;
-      if (C==1) {
-         mdct_backward(lookup, X, out_mem+C*(MAX_PERIOD-N-N4), window, overlap);
-      } else {
+      if (transient_shift==0 && C==1 && !shortBlocks) {
+         const mdct_lookup *lookup = MDCT(mode);
+         mdct_backward(lookup, X, out_mem+C*(MAX_PERIOD-N-N4), mode->window, overlap);
+      } else if (!shortBlocks) {
+         const mdct_lookup *lookup = MDCT(mode);
          VARDECL(celt_word32_t, x);
          VARDECL(celt_word32_t, tmp);
          SAVE_STACK;
@@ -197,7 +299,50 @@ static void compute_inv_mdcts(const CELTMode *mode, const celt_word16_t * restri
             tmp[j] = X[C*j+c];
          /* Prevents problems from the imdct doing the overlap-add */
          CELT_MEMSET(x+N4, 0, overlap);
-         mdct_backward(lookup, tmp, x, window, overlap);
+         mdct_backward(lookup, tmp, x, mode->window, overlap);
+         celt_assert(transient_shift == 0);
+         /* The first and last part would need to be set to zero if we actually
+            wanted to use them. */
+         for (j=0;j<overlap;j++)
+            out_mem[C*(MAX_PERIOD-N)+C*j+c] += x[j+N4];
+         for (j=0;j<overlap;j++)
+            out_mem[C*(MAX_PERIOD)+C*(overlap-j-1)+c] = x[2*N-j-N4-1];
+         for (j=0;j<2*N4;j++)
+            out_mem[C*(MAX_PERIOD-N)+C*(j+overlap)+c] = x[j+N4+overlap];
+         RESTORE_STACK;
+      } else {
+         int b;
+         const int N2 = mode->shortMdctSize;
+         const int B = mode->nbShortMdcts;
+         const mdct_lookup *lookup = &mode->shortMdct;
+         VARDECL(celt_word32_t, x);
+         VARDECL(celt_word32_t, tmp);
+         SAVE_STACK;
+         ALLOC(x, 2*N, celt_word32_t);
+         ALLOC(tmp, N, celt_word32_t);
+         /* Prevents problems from the imdct doing the overlap-add */
+         CELT_MEMSET(x+N4, 0, overlap);
+         for (b=0;b<B;b++)
+         {
+            /* De-interleaving the sub-frames */
+            for (j=0;j<N2;j++)
+               tmp[j] = X[C*(j*B+b)+c];
+            mdct_backward(lookup, tmp, x+N4+N2*b, mode->window, overlap);
+         }
+         if (transient_shift > 0)
+         {
+#ifdef FIXED_POINT
+            for (j=0;j<16;j++)
+               x[N4+transient_time+j-16] = MULT16_32_Q15(SHR16(Q15_ONE-transientWindow[j],transient_shift)+transientWindow[j], SHL32(x[N4+transient_time+j-16],transient_shift));
+            for (j=transient_time;j<N+overlap;j++)
+               x[N4+j] = SHL32(x[N4+j], transient_shift);
+#else
+            for (j=0;j<16;j++)
+               x[N4+transient_time+j-16] *= 1+transientWindow[j]*((1<<transient_shift)-1);
+            for (j=transient_time;j<N+overlap;j++)
+               x[N4+j] *= 1<<transient_shift;
+#endif
+         }
          /* The first and last part would need to be set to zero if we actually
          wanted to use them. */
          for (j=0;j<overlap;j++)
@@ -211,7 +356,7 @@ static void compute_inv_mdcts(const CELTMode *mode, const celt_word16_t * restri
    }
 }
 
-int EXPORT celt_encode(CELTEncoder * restrict st, celt_int16_t * restrict pcm, unsigned char *compressed, int nbCompressedBytes)
+int celt_encode(CELTEncoder * restrict st, celt_int16_t * restrict pcm, unsigned char *compressed, int nbCompressedBytes)
 {
    int i, c, N, N4;
    int has_pitch;
@@ -223,6 +368,13 @@ int EXPORT celt_encode(CELTEncoder * restrict st, celt_int16_t * restrict pcm, u
    VARDECL(celt_norm_t, P);
    VARDECL(celt_ener_t, bandE);
    VARDECL(celt_pgain_t, gains);
+   VARDECL(int, stereo_mode);
+#ifdef EXP_PSY
+   VARDECL(celt_word32_t, mask);
+#endif
+   int shortBlocks=0;
+   int transient_time;
+   int transient_shift;
    const int C = CHANNELS(st->mode);
    SAVE_STACK;
 
@@ -249,23 +401,74 @@ int EXPORT celt_encode(CELTEncoder * restrict st, celt_int16_t * restrict pcm, u
       }
    }
    CELT_COPY(st->in_mem, in+C*(2*N-2*N4-st->overlap), C*st->overlap);
-
+   
+   if (st->mode->nbShortMdcts > 1)
+   {
+      if (transient_analysis(in, N+st->overlap, C, &transient_time, &transient_shift))
+      {
+#ifndef FIXED_POINT
+         float gain_1;
+#endif
+         ec_enc_bits(&st->enc, 1, 1);
+         ec_enc_bits(&st->enc, transient_shift, 2);
+         if (transient_shift)
+            ec_enc_uint(&st->enc, transient_time, N+st->overlap);
+         if (transient_shift)
+         {
+#ifdef FIXED_POINT
+            for (c=0;c<C;c++)
+               for (i=0;i<16;i++)
+                  in[C*(transient_time+i-16)+c] = MULT16_32_Q15(EXTRACT16(SHR32(celt_rcp(Q15ONE+MULT16_16(transientWindow[i],((1<<transient_shift)-1))),1)), in[C*(transient_time+i-16)+c]);
+            for (c=0;c<C;c++)
+               for (i=transient_time;i<N+st->overlap;i++)
+                  in[C*i+c] = SHR32(in[C*i+c], transient_shift);
+#else
+            for (c=0;c<C;c++)
+               for (i=0;i<16;i++)
+                  in[C*(transient_time+i-16)+c] /= 1+transientWindow[i]*((1<<transient_shift)-1);
+            gain_1 = 1./(1<<transient_shift);
+            for (c=0;c<C;c++)
+               for (i=transient_time;i<N+st->overlap;i++)
+                  in[C*i+c] *= gain_1;
+#endif
+         }
+         shortBlocks = 1;
+      } else {
+         ec_enc_bits(&st->enc, 0, 1);
+         transient_time = -1;
+         transient_shift = 0;
+         shortBlocks = 0;
+      }
+   } else {
+      transient_time = -1;
+      transient_shift = 0;
+      shortBlocks = 0;
+   }
    /* Pitch analysis: we do it early to save on the peak stack space */
-   find_spectral_pitch(st->mode, st->mode->fft, &st->mode->psy, in, st->out_mem, st->mode->window, 2*N-2*N4, MAX_PERIOD-(2*N-2*N4), &pitch_index);
+   if (!shortBlocks)
+      find_spectral_pitch(st->mode, st->mode->fft, &st->mode->psy, in, st->out_mem, st->mode->window, 2*N-2*N4, MAX_PERIOD-(2*N-2*N4), &pitch_index);
 
    ALLOC(freq, C*N, celt_sig_t); /**< Interleaved signal MDCTs */
    
    /*for (i=0;i<(B+1)*C*N;i++) printf ("%f(%d) ", in[i], i); printf ("\n");*/
    /* Compute MDCTs */
-   compute_mdcts(st->mode, st->mode->window, in, freq);
+   compute_mdcts(st->mode, shortBlocks, in, freq);
 
-#if 0 /* Mask disabled until it can be made to do something useful */
-   compute_mdct_masking(X, mask, B*C*N, st->Fs);
+#ifdef EXP_PSY
+   CELT_MOVE(st->psy_mem, st->out_mem+N, MAX_PERIOD+st->overlap-N);
+   for (i=0;i<N;i++)
+      st->psy_mem[MAX_PERIOD+st->overlap-N+i] = in[C*(st->overlap+i)];
+   for (c=1;c<C;c++)
+      for (i=0;i<N;i++)
+         st->psy_mem[MAX_PERIOD+st->overlap-N+i] += in[C*(st->overlap+i)+c];
+
+   ALLOC(mask, N, celt_sig_t);
+   compute_mdct_masking(&st->psy, freq, st->psy_mem, mask, C*N);
 
    /* Invert and stretch the mask to length of X 
       For some reason, I get better results by using the sqrt instead,
       although there's no valid reason to. Must investigate further */
-   for (i=0;i<B*C*N;i++)
+   for (i=0;i<C*N;i++)
       mask[i] = 1/(.1+mask[i]);
 #endif
    
@@ -290,7 +493,8 @@ int EXPORT celt_encode(CELTEncoder * restrict st, celt_int16_t * restrict pcm, u
    /*for (i=0;i<N*B*C;i++)printf("%f ", X[i]);printf("\n");*/
 
    /* Compute MDCTs of the pitch part */
-   compute_mdcts(st->mode, st->mode->window, st->out_mem+pitch_index*C, freq);
+   if (!shortBlocks)
+      compute_mdcts(st->mode, 0, st->out_mem+pitch_index*C, freq);
 
    {
       /* Normalise the pitch vector as well (discard the energies) */
@@ -302,7 +506,7 @@ int EXPORT celt_encode(CELTEncoder * restrict st, celt_int16_t * restrict pcm, u
    }
    curr_power = bandE[0]+bandE[1]+bandE[2];
    /* Check if we can safely use the pitch (i.e. effective gain isn't too high) */
-   if (MULT16_32_Q15(QCONST16(.1f, 15),curr_power) + QCONST32(10.f,ENER_SHIFT) < pitch_power)
+   if (!shortBlocks && (MULT16_32_Q15(QCONST16(.1f, 15),curr_power) + QCONST32(10.f,ENER_SHIFT) < pitch_power))
    {
       /* Simulates intensity stereo */
       /*for (i=30;i<N*B;i++)
@@ -321,27 +525,20 @@ int EXPORT celt_encode(CELTEncoder * restrict st, celt_int16_t * restrict pcm, u
       for (i=0;i<C*N;i++)
          P[i] = 0;
    }
-   quant_energy(st->mode, bandE, st->oldBandE, 20+nbCompressedBytes*8/5, st->mode->prob, &st->enc);
+   quant_energy(st->mode, bandE, st->oldBandE, 20*C+nbCompressedBytes*8/5, st->mode->prob, &st->enc);
 
-   if (C==2)
-   {
-      stereo_mix(st->mode, X, bandE, 1);
-      stereo_mix(st->mode, P, bandE, 1);
-   }
+   ALLOC(stereo_mode, st->mode->nbEBands, int);
+   stereo_decision(st->mode, X, stereo_mode, st->mode->nbEBands);
 
    pitch_quant_bands(st->mode, P, gains);
 
    /*for (i=0;i<B*N;i++) printf("%f ",P[i]);printf("\n");*/
-   /* Compute residual that we're going to encode */
-   for (i=0;i<C*st->mode->eBands[st->mode->nbEBands];i++)
-      X[i] -= P[i];
 
    /* Residual quantisation */
-   quant_bands(st->mode, X, P, NULL, nbCompressedBytes*8, &st->enc);
+   quant_bands(st->mode, X, P, NULL, bandE, stereo_mode, nbCompressedBytes*8, shortBlocks, &st->enc);
    
    if (C==2)
    {
-      stereo_mix(st->mode, X, bandE, -1);
       renormalise_bands(st->mode, X);
    }
    /* Synthesis */
@@ -350,13 +547,13 @@ int EXPORT celt_encode(CELTEncoder * restrict st, celt_int16_t * restrict pcm, u
 
    CELT_MOVE(st->out_mem, st->out_mem+C*N, C*(MAX_PERIOD+st->overlap-N));
 
-   compute_inv_mdcts(st->mode, st->mode->window, freq, st->out_mem);
+   compute_inv_mdcts(st->mode, shortBlocks, freq, transient_time, transient_shift, st->out_mem);
    /* De-emphasis and put everything back at the right place in the synthesis history */
 #ifndef SHORTCUTS
    for (c=0;c<C;c++)
    {
       int j;
-      const celt_sig_t * restrict outp=st->out_mem+C*(MAX_PERIOD-N)+c;
+      celt_sig_t * restrict outp=st->out_mem+C*(MAX_PERIOD-N)+c;
       celt_int16_t * restrict pcmp = pcm+c;
       for (j=0;j<N;j++)
       {
@@ -368,8 +565,8 @@ int EXPORT celt_encode(CELTEncoder * restrict st, celt_int16_t * restrict pcm, u
       }
    }
 #endif
-   if (ec_enc_tell(&st->enc, 0) < nbCompressedBytes*8 - 7)
-      celt_warning_int ("many unused bits: ", nbCompressedBytes*8-ec_enc_tell(&st->enc, 0));
+   /*if (ec_enc_tell(&st->enc, 0) < nbCompressedBytes*8 - 7)
+      celt_warning_int ("many unused bits: ", nbCompressedBytes*8-ec_enc_tell(&st->enc, 0));*/
    /*printf ("%d\n", ec_enc_tell(&st->enc, 0)-8*nbCompressedBytes);*/
    /* Finishing the stream with a 0101... pattern so that the decoder can check is everything's right */
    {
@@ -434,7 +631,7 @@ struct CELTDecoder {
    int last_pitch_index;
 };
 
-CELTDecoder EXPORT *celt_decoder_create(const CELTMode *mode)
+CELTDecoder *celt_decoder_create(const CELTMode *mode)
 {
    int N, C;
    CELTDecoder *st;
@@ -461,7 +658,7 @@ CELTDecoder EXPORT *celt_decoder_create(const CELTMode *mode)
    return st;
 }
 
-void EXPORT celt_decoder_destroy(CELTDecoder *st)
+void celt_decoder_destroy(CELTDecoder *st)
 {
    if (st == NULL)
    {
@@ -508,7 +705,7 @@ static void celt_decode_lost(CELTDecoder * restrict st, short * restrict pcm)
    offset = MAX_PERIOD-pitch_index;
    while (offset+len >= MAX_PERIOD)
       offset -= pitch_index;
-   compute_mdcts(st->mode, st->mode->window, st->out_mem+offset*C, freq);
+   compute_mdcts(st->mode, 0, st->out_mem+offset*C, freq);
    for (i=0;i<N;i++)
       freq[i] = MULT16_32_Q15(QCONST16(.9f,15),freq[i]);
 #endif
@@ -517,7 +714,7 @@ static void celt_decode_lost(CELTDecoder * restrict st, short * restrict pcm)
    
    CELT_MOVE(st->out_mem, st->out_mem+C*N, C*(MAX_PERIOD+st->mode->overlap-N));
    /* Compute inverse MDCTs */
-   compute_inv_mdcts(st->mode, st->mode->window, freq, st->out_mem);
+   compute_inv_mdcts(st->mode, 0, freq, -1, 1, st->out_mem);
 
    for (c=0;c<C;c++)
    {
@@ -533,7 +730,7 @@ static void celt_decode_lost(CELTDecoder * restrict st, short * restrict pcm)
    RESTORE_STACK;
 }
 
-int EXPORT celt_decode(CELTDecoder * restrict st, unsigned char *data, int len, celt_int16_t * restrict pcm)
+int celt_decode(CELTDecoder * restrict st, unsigned char *data, int len, celt_int16_t * restrict pcm)
 {
    int c, N, N4;
    int has_pitch;
@@ -545,6 +742,10 @@ int EXPORT celt_decode(CELTDecoder * restrict st, unsigned char *data, int len,
    VARDECL(celt_norm_t, P);
    VARDECL(celt_ener_t, bandE);
    VARDECL(celt_pgain_t, gains);
+   VARDECL(int, stereo_mode);
+   int shortBlocks;
+   int transient_time;
+   int transient_shift;
    const int C = CHANNELS(st->mode);
    SAVE_STACK;
 
@@ -575,6 +776,25 @@ int EXPORT celt_decode(CELTDecoder * restrict st, unsigned char *data, int len,
    ec_byte_readinit(&buf,data,len);
    ec_dec_init(&dec,&buf);
    
+   if (st->mode->nbShortMdcts > 1)
+   {
+      shortBlocks = ec_dec_bits(&dec, 1);
+      if (shortBlocks)
+      {
+         transient_shift = ec_dec_bits(&dec, 2);
+         if (transient_shift)
+            transient_time = ec_dec_uint(&dec, N+st->mode->overlap);
+         else
+            transient_time = 0;
+      } else {
+         transient_time = -1;
+         transient_shift = 0;
+      }
+   } else {
+      shortBlocks = 0;
+      transient_time = -1;
+      transient_shift = 0;
+   }
    /* Get the pitch gains */
    has_pitch = unquant_pitch(gains, st->mode->nbPBands, &dec);
    
@@ -589,10 +809,10 @@ int EXPORT celt_decode(CELTDecoder * restrict st, unsigned char *data, int len,
    }
 
    /* Get band energies */
-   unquant_energy(st->mode, bandE, st->oldBandE, 20+len*8/5, st->mode->prob, &dec);
+   unquant_energy(st->mode, bandE, st->oldBandE, 20*C+len*8/5, st->mode->prob, &dec);
 
    /* Pitch MDCT */
-   compute_mdcts(st->mode, st->mode->window, st->out_mem+pitch_index*C, freq);
+   compute_mdcts(st->mode, 0, st->out_mem+pitch_index*C, freq);
 
    {
       VARDECL(celt_ener_t, bandEp);
@@ -601,18 +821,16 @@ int EXPORT celt_decode(CELTDecoder * restrict st, unsigned char *data, int len,
       normalise_bands(st->mode, freq, P, bandEp);
    }
 
-   if (C==2)
-      stereo_mix(st->mode, P, bandE, 1);
-
+   ALLOC(stereo_mode, st->mode->nbEBands, int);
+   stereo_decision(st->mode, X, stereo_mode, st->mode->nbEBands);
    /* Apply pitch gains */
    pitch_quant_bands(st->mode, P, gains);
 
    /* Decode fixed codebook and merge with pitch */
-   unquant_bands(st->mode, X, P, len*8, &dec);
+   unquant_bands(st->mode, X, P, bandE, stereo_mode, len*8, shortBlocks, &dec);
 
    if (C==2)
    {
-      stereo_mix(st->mode, X, bandE, -1);
       renormalise_bands(st->mode, X);
    }
    /* Synthesis */
@@ -621,7 +839,7 @@ int EXPORT celt_decode(CELTDecoder * restrict st, unsigned char *data, int len,
 
    CELT_MOVE(st->out_mem, st->out_mem+C*N, C*(MAX_PERIOD+st->overlap-N));
    /* Compute inverse MDCTs */
-   compute_inv_mdcts(st->mode, st->mode->window, freq, st->out_mem);
+   compute_inv_mdcts(st->mode, shortBlocks, freq, transient_time, transient_shift, st->out_mem);
 
    for (c=0;c<C;c++)
    {
