@@ -52,7 +52,7 @@
 #endif
 
 
-int EXPORT celt_mode_info(const CELTMode *mode, int request, celt_int32_t *value)
+int celt_mode_info(const CELTMode *mode, int request, celt_int32_t *value)
 {
    switch (request)
    {
@@ -209,18 +209,24 @@ static void compute_allocation_table(CELTMode *mode, int res)
       for (j=0;j<BARK_BANDS;j++)
       {
          int edge, low;
+         celt_int32_t alloc;
          edge = mode->eBands[eband+1]*res;
+         alloc = band_allocation[i*BARK_BANDS+j];
+         if (mode->nbChannels == 2)
+            alloc = alloc*3*mode->mdctSize/512;
+         else
+            alloc = alloc*mode->mdctSize/256;
          if (edge < bark_freq[j+1])
          {
             int num, den;
-            num = band_allocation[i*BARK_BANDS+j] * (edge-bark_freq[j]);
+            num = alloc * (edge-bark_freq[j]);
             den = bark_freq[j+1]-bark_freq[j];
             low = (num+den/2)/den;
             allocVectors[i*mode->nbEBands+eband] += low;
             eband++;
-            allocVectors[i*mode->nbEBands+eband] += band_allocation[i*BARK_BANDS+j]-low;
+            allocVectors[i*mode->nbEBands+eband] += alloc-low;
          } else {
-            allocVectors[i*mode->nbEBands+eband] += band_allocation[i*BARK_BANDS+j];
+            allocVectors[i*mode->nbEBands+eband] += alloc;
          }
       }
    }
@@ -267,7 +273,7 @@ static void compute_energy_allocation_table(CELTMode *mode)
    mode->energy_alloc = alloc;
 }
 
-CELTMode EXPORT *celt_mode_create(celt_int32_t Fs, int channels, int frame_size, int lookahead, int *error)
+CELTMode *celt_mode_create(celt_int32_t Fs, int channels, int frame_size, int *error)
 {
    int i;
 #ifdef STDIN_TUNING
@@ -325,16 +331,9 @@ CELTMode EXPORT *celt_mode_create(celt_int32_t Fs, int channels, int frame_size,
          *error = CELT_BAD_ARG;
       return NULL;
    }
-   if (frame_size < 64 || frame_size > 256 || frame_size%2!=0)
+   if (frame_size < 64 || frame_size > 512 || frame_size%2!=0)
    {
-      celt_warning("Only even frame sizes between 64 and 256 are supported");
-      if (error)
-         *error = CELT_BAD_ARG;
-      return NULL;
-   }
-   if (lookahead < 32 || lookahead > frame_size)
-   {
-      celt_warning("The overlap must be between 32 and the frame size");
+      celt_warning("Only even frame sizes between 64 and 512 are supported");
       if (error)
          *error = CELT_BAD_ARG;
       return NULL;
@@ -343,13 +342,29 @@ CELTMode EXPORT *celt_mode_create(celt_int32_t Fs, int channels, int frame_size,
    
    mode = celt_alloc(sizeof(CELTMode));
    mode->Fs = Fs;
-   mode->overlap = lookahead;
    mode->mdctSize = frame_size;
    mode->nbChannels = channels;
    mode->eBands = compute_ebands(Fs, frame_size, &mode->nbEBands);
    compute_pbands(mode, res);
    mode->ePredCoef = QCONST16(.8f,15);
    
+   if (frame_size <= 64)
+   {
+      mode->nbShortMdcts = 1;
+   } else if (frame_size <= 256)
+   {
+      mode->nbShortMdcts = 2;
+   } else if (frame_size <= 384)
+   {
+      mode->nbShortMdcts = 3;
+   } else {
+      mode->nbShortMdcts = 4;
+   }
+   if (mode->nbShortMdcts > 1)
+      mode->overlap = frame_size/mode->nbShortMdcts;
+   else
+      mode->overlap = frame_size/2;
+   
    compute_allocation_table(mode, res);
    /*printf ("%d bands\n", mode->nbEBands);*/
    
@@ -364,8 +379,9 @@ CELTMode EXPORT *celt_mode_create(celt_int32_t Fs, int channels, int frame_size,
 #endif
    mode->window = window;
 
-   compute_alloc_cache(mode);
+   mode->bits = (const celt_int16_t **)compute_alloc_cache(mode, 1);
 
+   mode->bits_stereo = NULL;
 #ifndef SHORTCUTS
    psydecay_init(&mode->psy, MAX_PERIOD/2, mode->Fs);
 #endif
@@ -376,15 +392,22 @@ CELTMode EXPORT *celt_mode_create(celt_int32_t Fs, int channels, int frame_size,
    mdct_init(&mode->mdct, 2*mode->mdctSize);
    mode->fft = pitch_state_alloc(MAX_PERIOD);
 
+   mode->shortMdctSize = mode->mdctSize/mode->nbShortMdcts;
+   mdct_init(&mode->shortMdct, 2*mode->shortMdctSize);
+   mode->shortWindow = mode->window;
+
    mode->prob = quant_prob_alloc(mode);
    compute_energy_allocation_table(mode);
    
+   if (mode->nbChannels>=2)
+      mode->bits_stereo = (const celt_int16_t **)compute_alloc_cache(mode, mode->nbChannels);
+
    if (error)
       *error = CELT_OK;
    return mode;
 }
 
-void EXPORT celt_mode_destroy(CELTMode *mode)
+void celt_mode_destroy(CELTMode *mode)
 {
 #ifndef STATIC_MODES
    int i;
